{% extends "main::container.html" %}

{% block article %}
<article class="nav">
    <section class="row">
        <nav>
            {% for textgroup in textgroups|order_author %}
                <div><a href="{{url_for('.r_texts', collection=textgroup.urn.namespace, textgroup=textgroup.urn.textgroup)}}">{{ textgroup.metadata['groupname'][lang] }}</a></div>
            {% endfor %}
        </nav>
    </section>
</article>
{% endblock %}