{% extends "main::container.html" %}

{% import "main::reference_display.html" as reference_display %}

{% block article %}
<article class="nav reffs">
    <section class="row">
            {{ reference_display.hierarchical_dispatcher(version, reffs) }}
    </section>
</article>
{% endblock %}